unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, Buttons,
  Arrow, StdCtrls, Types, Math;

type

  { TForm1 }
  TArrowType = (atUp, atDown, atLeft, atRight);

  TForm1 = class(TForm)
     cboGlyphKind: TComboBox;
     CheckBox1: TCheckBox;
     CheckBox2: TCheckBox;
     Label3: TLabel;
     FMyGlyphUP: TBitmap;
     FMyGlyphDown: TBitmap;
     FMyGlyphLeft: TBitmap;
     FMyGlyphRight: TBitmap;
     SpeedButton1: TSpeedButton;
     SpeedButton2: TSpeedButton;
     procedure cboGlyphKindChange(Sender: TObject);
     procedure CheckBox1Change(Sender: TObject);
     procedure CheckBox2Change(Sender: TObject);
     procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
     procedure FormCreate(Sender: TObject);
     procedure SpeedButton1MouseEnter(Sender: TObject);
     procedure SpeedButton1MouseLeave(Sender: TObject);
  private
     function MakeArrow (iWidth, iHeight: Integer; ArrowType: TArrowType): TBitmap;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
   FMyGlyphUp               := MakeArrow (SpeedButton1.Width, SpeedButton1.Height, atUp   );
   FMyGlyphDown             := MakeArrow (SpeedButton1.Width, SpeedButton1.Height, atDown );
   FMyGlyphLeft             := MakeArrow (SpeedButton1.Width, SpeedButton1.Height, atLeft );
   FMyGlyphRight            := MakeArrow (SpeedButton1.Width, SpeedButton1.Height, atRight);
   cboGlyphKind.ItemIndex   := 1;
   CheckBox1.Checked        := True;
   CheckBox2.Checked        := True;
   SpeedButton1.Glyph       := FMyGlyphUp
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
   FreeAndNil (FMyGlyphUp   );
   FreeAndNil (FMyGlyphDown );
   FreeAndNil (FMyGlyphLeft );
   FreeAndNil (FMyGlyphRight)
end;

procedure TForm1.SpeedButton1MouseEnter(Sender: TObject);
begin
   TSpeedButton(Sender).Down := True
end;

procedure TForm1.SpeedButton1MouseLeave(Sender: TObject);
begin
   TSpeedButton(Sender).Down := False
end;

procedure TForm1.cboGlyphKindChange(Sender: TObject);
begin
   case cboGlyphKind.ItemIndex of
      0:  SpeedButton1.Glyph := NIL          ;
      1:  SpeedButton1.Glyph := FMyGlyphUp   ;
      2:  SpeedButton1.Glyph := FMyGlyphDown ;
      3:  SpeedButton1.Glyph := FMyGlyphLeft ;
      4:  SpeedButton1.Glyph := FMyGlyphRight
   end
end;

procedure TForm1.CheckBox1Change(Sender: TObject);
begin
   speedbutton1.Flat:= CheckBox1.Checked;
   speedbutton2.Flat:= CheckBox1.Checked;
end;

procedure TForm1.CheckBox2Change(Sender: TObject);
begin
   speedbutton1.Transparent:= CheckBox2.Checked;
   speedbutton2.Transparent:= CheckBox2.Checked;
end;

function TForm1.MakeArrow (iWidth, iHeight: Integer; ArrowType: TArrowType): TBitmap;
// The arrow has a right-angled tip
const cInnerOffset = 8;
var BM        : TBitmap;
    FR        : TRect;
    FT        : array[1 .. 3] of TPoint;
    midY, midX: integer;
    ratioNeed : double;
    size      : TSize;
begin
   BM                  := TBitMap.Create;
   BM.Transparent      := true;
   BM.TransparentColor := clFuchsia;
   BM.TransparentMode  := tmFixed;
   FR                  := Rect (cInnerOffset, cInnerOffset, iWidth - cInnerOffset, iHeight - cInnerOffset);
   size                := Types.Size(FR);
   midX                := (FR.Left + FR.Right ) div 2;
   midY                := (FR.Top  + FR.Bottom) div 2;

   if ArrowType in [atLeft, atRight] then  ratioNeed := 0.5
                                     else  ratioNeed := 2;
   if ((size.cx/size.cy) >= ratioNeed) then  size.cx := Trunc(size.cy*ratioNeed)
                                       else  size.cy := Trunc(size.cx/ratioNeed);

   if ArrowType in [atUp, atDown] then begin
      FR.Left   := midX   - size.cy;
      FR.Right  := midX   + size.cy;
      FR.Top    := midY   - size.cy div 2;
      FR.Bottom := FR.Top + size.cy      ;
   end
   else begin
      FR.Left   := midX    - size.cx div 2;
      FR.Right  := FR.Left + size.cx      ;
      FR.Top    := midY    - size.cx;
      FR.Bottom := midY    + size.cx;
   end;

   FR.Right  := FR.right - FR.Left;
   FR.Left   := 0 ;
   FR.Bottom := FR.Bottom - FR.Top;
   FR.Top    := 0;
   midx      := FR.Right  div 2;
   midy      := FR.Bottom div 2;
   BM.Width  := FR.Right;
   BM.Height := FR.Bottom + 1;

   case ArrowType of
     atUp: begin
         FT[1] := Point(FR.Left, FR.Bottom);
         FT[2] := FR.BottomRight;
         FT[3] := Point(midX, FR.Top);
        end;
     atDown: begin
         FT[1] := FR.TopLeft;
         FT[2] := Point(FR.Right, FR.Top);
         FT[3] := Point(midX, FR.Bottom);
        end;
     atLeft: begin
         FT[1] := Point(FR.Right, FR.Top);
         FT[2] := FR.BottomRight;
         FT[3] := Point(FR.Left, midY);
        end;
     atRight: begin
         FT[1] := FR.TopLeft;
         FT[2] := Point(FR.Right, midY);
         FT[3] := Point(FR.Left, FR.Bottom);
        end;
   end;

   BM.Canvas.Brush.Color := clFuchsia;
   BM.Canvas.FillRect(BM.Canvas.ClipRect);

   BM.Canvas.Pen.Color   := clGray;
   BM.Canvas.Brush.Color := clGray;
   BM.Canvas.Polygon (FT);
   Result := BM
end;

end.

